from __future__ import absolute_import

try:
    import py7zlib
except ImportError:
    pass

import logging
from io import BytesIO

from .codec import Codec

LOGGER = logging.getLogger(__name__)


class SevenZ(Codec):
    def __init__(self):
        if 'py7zlib' not in globals():
            LOGGER.warn('py7zlib could not be imported. To read 7z files, '
                        'install the library with "pip install pylzma".')

    def compress(self, stream):
        LOGGER.warn('Writing of 7z compressed archives is not supported.')
        return stream

    def decompress(self, stream):
        if 'py7zlib' not in globals():
            return Codec.decompress(self, stream)

        uncompressed = BytesIO()

        f = py7zlib.Archive7z(file=stream)
        for f_name in f.getnames():
            uncompressed.write(f.getmember(f_name).read())

        uncompressed.seek(0)
        return uncompressed



import logging

log = logging.getLogger(__name__)


class FileSystem(object):
    def __init__(self, file_name):
        self.file_name = file_name

    @staticmethod
    def resolve_filenames(expr):
        log.error('Cannot resolve: {0}'.format(expr))

    def exists(self):
        log.warning('Could not determine whether {0} exists due to '
                    'unhandled scheme.'.format(self.file_name))

    def load(self):
        log.error('Cannot load: {0}'.format(self.file_name))

    def dump(self, stream):
        log.error('Cannot dump: {0}'.format(self.file_name))

    def make_public(self, recursive=False):
        log.warning('Cannot make {0} public.'.format(self.file_name))



from __future__ import absolute_import

import logging
from io import BytesIO, StringIO

from .file_system import FileSystem
from ...exceptions import ConnectionException, FileSystemNotSupported

log = logging.getLogger(__name__)

requests = None
try:
    import requests
except ImportError:
    pass


class Http(FileSystem):
    def __init__(self, file_name):
        if requests is None:
            raise FileSystemNotSupported(
                'http not supported. Install "requests".'
            )

        FileSystem.__init__(self, file_name)
        self.headers = None

    @staticmethod
    def resolve_filenames(expr):
        if Http(expr).exists():
            return [expr]
        return []

    def exists(self):
        r = requests.head(self.file_name, allow_redirects=True)
        return r.status_code == 200

    def load(self):
        log.debug('Http GET request for {0}.'.format(self.file_name))
        r = requests.get(self.file_name, headers=self.headers)
        if r.status_code != 200:
            raise ConnectionException
        return BytesIO(r.content)

    def load_text(self):
        log.debug('Http GET request for {0}.'.format(self.file_name))
        r = requests.get(self.file_name, headers=self.headers)
        if r.status_code != 200:
            raise ConnectionException
        return StringIO(r.text)

    def dump(self, stream):
        log.debug('Dump to {0} with http PUT.'.format(self.file_name))
        requests.put(self.file_name, data=b''.join(stream))
        return self



from __future__ import absolute_import


# lzma only available in Python >= 3.3
try:
    import lzma
except ImportError:
    pass

import logging
from io import BytesIO

from .codec import Codec

LOGGER = logging.getLogger(__name__)


class Lzma(Codec):
    def __init__(self):
        if 'lzma' not in globals():
            LOGGER.warn('LZMA codec not supported. It is only supported '
                        'in Python>=3.3. Not compressing streams.')

    def compress(self, stream):
        if 'lzma' not in globals():
            return Codec.compress(self, stream)

        return BytesIO(lzma.compress(stream.read()))

    def decompress(self, stream):
        if 'lzma' not in globals():
            return Codec.decompress(self, stream)

        return BytesIO(lzma.decompress(stream.read()))
